#ifndef CStack_H
#define CStack_H
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define STACK_ITEM_TYPE_BINARY		0
#define STACK_ITEM_TYPE_UINT		1
#define STACK_ITEM_TYPE_INT			2
#define STACK_ITEM_TYPE_FLOAT		3
#define STACK_ITEM_TYPE_DOUBLE		4
#define STACK_ITEM_TYPE_STRING		5
#define STACK_ITEM_TYPE_WSTRING		6
#define STACK_ITEM_TYPE_INT64		7
#define STACK_ITEM_TYPE_UINT64		8

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Stack_Item {
	void *Value;
	size_t Size;
	unsigned short Type;
} STACKITEM, *LPSTACKITEM;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Stack {
	STACKITEM *Item;
	size_t Used;
	size_t Ceiling;
	size_t AllocationIncrement;
} STACK, *LPSTACK;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CStack {

public:
	CStack();
	CStack(CStack *pStackToCopy);
	~CStack();

	void Clear(void);

	bool Push(unsigned __int64 u64Value);
	bool Push(__int64 i64Value);
	bool Push(unsigned int uValue);
	bool Push(int iValue);
	bool Push(float fValue);
	bool Push(double dValue);
	bool Push(const char *sValue, size_t uLength);
	bool Push(const char *sValue);
	bool Push(const WCHAR *wcValue, size_t uLength);
	bool Push(const WCHAR *wcValue);
	bool Push(const void *pValue, size_t uLength, unsigned short uType);
	bool Push(const void *pValue, size_t uLength);

	bool Pop(unsigned __int64 *u64Value);
	bool Pop(__int64 *i64Value);
	bool Pop(unsigned int *uValue);
	bool Pop(int *iValue);
	bool Pop(float *fValue);
	bool Pop(double *dValue);
	bool Pop(char *sValue, size_t *uLength);
	bool Pop(WCHAR *wcValue, size_t *uLength);
	bool Pop(void *pValue);
	bool Pop(void *pValue, size_t *uLength);
	bool Pop(void *pValue, size_t *uLength, unsigned short *uType);

	STACKITEM *Peek(void);
	bool Peek(unsigned __int64 *u64Value);
	bool Peek(__int64 *i64Value);
	bool Peek(unsigned int *uValue);
	bool Peek(int *iValue);
	bool Peek(float *fValue);
	bool Peek(double *dValue);
	bool Peek(char *sValue, size_t *uLength);
	bool Peek(WCHAR *wcValue, size_t *uLength);
	bool Peek(void *pValue);
	bool Peek(void *pValue, size_t *uLength);
	bool Peek(void *pValue, size_t *uLength, unsigned short *uType);

	STACKITEM *Peek(size_t uIndex);
	bool Peek(size_t uIndex, unsigned __int64 *u64Value);
	bool Peek(size_t uIndex, __int64 *i64Value);
	bool Peek(size_t uIndex, unsigned int *uValue);
	bool Peek(size_t uIndex, int *iValue);
	bool Peek(size_t uIndex, float *fValue);
	bool Peek(size_t uIndex, double *dValue);
	bool Peek(size_t uIndex, char *sValue, size_t *uLength);
	bool Peek(size_t uIndex, WCHAR *wcValue, size_t *uLength);
	bool Peek(size_t uIndex, void *pValue);
	bool Peek(size_t uIndex, void *pValue, size_t *uLength);
	bool Peek(size_t uIndex, void *pValue, size_t *uLength, unsigned short *uType);

	size_t StackSize(void);
	size_t Allocated(void);

	void Lock(void);
	void UnLock(void);

private:
	STACK Stack;

	CRITICAL_SECTION CS;
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
